@@ -13,13 +13,15 @@
     N <- length(eY$which) * length(iY$which)
 
     if (is.null(eY)) {
-        Y <- iY$Yleft
+        Y <- Yt <- iY$Yleft
     } else {
-        Y <- eY$Y
+        Y <- Yt <- eY$Y
+        if (!is.null(iY)) Yt <- rbind(Yt, iY$Yleft)
     }
     Assign <- attr(Y, "Assign")
     SCALE <- "bscaling" %in% Assign[2,]
     if (SCALE) Z <- model.matrix(model$model$bscaling, data = data)
+    ### <FIXME> fixed current does not apply to scale terms in Z </FIXME>
 
     ui <- attr(Y, "constraint")$ui
     ci <- attr(Y, "constraint")$ci
@@ -29,10 +31,20 @@
         if (length(fixed) == 0) fixed <- NULL
     }
 
+    ### check for constant columns and fix corresponding parameters
+    const <- NULL
+    cYt <- which(colSums(abs(Yt)) < .Machine$double.eps)
+    if (length(cYt)) {
+        nf <- rep.int(0, length(cYt))
+        names(nf) <- colnames(Yt)[cYt]
+        fixed <- c(fixed, nf)
+    }
+
     if (!is.null(fixed)) {
-        stopifnot(all(names(fixed) %in% colnames(Y)))
-        fix <- colnames(Y) %in% names(fixed)
-        fixed <- fixed[colnames(Y)[fix]]
+        if (!all(names(fixed) %in% colnames(Yt)))
+            stop("Fixing parameters failed, maybe they apply to scale terms?")
+        fix <- colnames(Yt) %in% names(fixed)
+        fixed <- fixed[colnames(Yt)[fix]]
 
         ### adjust contrasts a fixed parameter contributes to
         ci <- ci - c(as(ui[, fix, drop = FALSE], "matrix") %*% fixed)
@@ -413,7 +425,9 @@
         }
 
         he = function(beta) {
-             ret <- 0
+            if ("bscaling" %in% Assign[2,])
+                stop("Analytical Hessian not available for location-scale models")
+            ret <- 0
             if (is.matrix(beta)) {
                 beta_ex <- beta[es$full_ex,,drop = FALSE]
                 beta_nex <- beta[es$full_nex,,drop = FALSE]
@@ -465,9 +479,10 @@
     }
 
     optimfct <- function(theta, weights, subset = NULL, offset = NULL, 
-                         scale = FALSE, optim, ...) {
+                         scaleparm = TRUE, optim, ...) {
         of <- .ofuns(weights = weights, subset = subset, 
                      offset = offset, ...)
+        inweights <- weights
 
         ### N contributions to the log-likelihood, UNWEIGHTED
         logliki <- function(beta, weights = NULL)
@@ -485,7 +500,7 @@
         hessian <- function(beta, weights) 
             .ofuns(weights = weights, offset = offset)$he(beta)
 
-        if (scale) {
+        if (scaleparm) {
             Ytmp <- Y
             Ytmp[!is.finite(Ytmp)] <- NA
             sc <- apply(abs(Ytmp[, !fix, drop = FALSE]), 2, max, na.rm = TRUE)
@@ -505,17 +520,26 @@
                 ## ret[names(sc)] <- ret[names(sc)] * sc
                 ret
             }
+            h <- function(gamma) {
+                hessian(gamma * sc, weights) * sc^2
+            }
             theta <- theta / sc
             if (!is.null(ui))
                 ui <- t(t(ui) * sc)
         } else {
             f <- function(gamma) loglikfct(gamma, weights)
             g <- function(gamma) scorefct(gamma, weights)
+            h <- function(gamma) hessian(gamma, weights)
         }
 
         if (dofit) {
             for (i in 1:length(optim)) {
-                ret <- optim[[i]](theta, f, g, ui, ci)
+                if (i > 1) {
+                    msg <- paste(names(optim)[i - 1], "did not converge, trying", names(optim)[i], sep = " ")
+                    warning(msg)
+                }	
+                ret <- optim[[i]](theta = theta, f = f, g = g, ui = ui, ci = ci, 
+                                  h = h)
                 if (ret$convergence == 0) break()
             }
         } else {
@@ -530,12 +554,26 @@
 #        if (!is.null(ui)) 
 #            ret$df <- ret$df - sum(ui %*% ret$par - ci < .Machine$double.eps)
         ### </FIXME>
-        if (scale) ret$par <- ret$par * sc
+        if (scaleparm) ret$par <- ret$par * sc
 
         if (SCALE) {
             ret$hessian <- function(beta, weights) {
-                # warning("Analytical Hessian not available, using numerical approximation")
-                ret <- numDeriv::hessian(loglikfct, beta, weights = weights)
+                H <- ret$optim_hessian
+                if (!scaleparm && !is.null(H) && 
+                    max(c(abs(beta - ret$par), 
+                          abs(weights - inweights))) < .Machine$double.eps) {
+                    ret <- H
+                } else {
+                    # warning("Analytical Hessian not available, using numerical approximation")
+                    ret <- try(stats::optimHess(ret$par, fn = loglikfct, gr = scorefct, 
+                                                weights = weights))
+                    ### this may fail because numDeriv can't deal with -Inf
+                    ### values of the target function
+                    if (inherits(ret, "try-error"))
+                        ret <- try(numDeriv::hessian(loglikfct, beta, weights = weights))
+                    if (inherits(ret, "try-error"))
+                        stop("Approximation of Hessian failed")
+                }
                 rownames(ret) <- colnames(ret) <- names(beta)
                 return(ret)
             }
@@ -559,7 +597,7 @@
         ret$trafo <- function(beta, weights) of$trafo(beta)
         ret$trafoprime <- function(beta, weights) of$trafoprime(beta)
 
-        if (scale) ret$parsc <- sc
+        if (scaleparm) ret$parsc <- sc
 
         return(ret)
     }
@@ -641,10 +679,16 @@
     }
 
     todistr <- model$todistr
-    Z <- todistr$q(pmax(.01, pmin(pstart, .99))) - offset
+    ### <TH> this makes sure we obtain the ML solution
+    ###      under a completely random outcome, ie the correct
+    ###      solution for an unconditional model
+    Z <- todistr$q(pmin(0, log(pstart)), log.p = TRUE) - offset
 
     X <- X * sqrt(weights)
     Z <- Z * sqrt(weights)
+    X <- X[is.finite(Z),,drop = FALSE]
+    Z <- Z[is.finite(Z)]
+    ### </TH>
 
     dvec <- crossprod(X, Z)
     Dmat <- crossprod(X)
@@ -694,14 +738,14 @@
 }
 
 .mlt_fit <- function(object, weights, subset = NULL, offset = NULL, 
-                     theta = NULL, scale = FALSE, optim, fixed = NULL, ...) {
+                     theta = NULL, scaleparm = TRUE, optim, fixed = NULL, ...) {
 
     if (is.null(theta))
         stop(sQuote("mlt"), "needs suitable starting values")
 
     ### BBoptim issues a warning in case of unsuccessful convergence
     ret <- try(object$optimfct(theta, weights = weights, 
-        subset = subset, offset = offset, scale = scale, 
+        subset = subset, offset = offset, scaleparm = scaleparm, 
         optim = optim, ...))    
 
     cls <- class(object)
@@ -710,7 +754,7 @@
     object$coef[] <- object$parm(ret$par) ### [] preserves names
     object$theta <- theta ### starting value
     object$subset <- subset
-    object$scale <- scale ### scaling yes/no
+    object$scaleparm <- scaleparm ### scaling yes/no
     object$weights <- weights
     object$offset <- offset
     object$optim <- optim
@@ -720,8 +764,8 @@
 }
 
 mlt <- function(model, data, weights = NULL, offset = NULL, fixed = NULL,
-                theta = NULL, pstart = NULL, scale = FALSE,
-                dofit = TRUE, optim = mltoptim()) {
+                theta = NULL, pstart = NULL, scaleparm = TRUE,
+                dofit = TRUE, optim = mltoptim(hessian = has_scale(model))) {
 
     vars <- as.vars(model)
     response <- variable.names(model, "response")
@@ -754,10 +798,10 @@ mlt <- function(model, data, weights = NULL, offset = NULL, fixed = NULL,
         ### unconditional ECDF, essentially
         ### this doesn't really work for censored data, any alternative?
         if (is.null(pstart)) 
-            pstart <- attr(y, "prob")(weights)(y$approxy) ### y$rank / max(y$rank)
+            pstart <- pstart(data[[response]], weights = weights)
         theta <- .mlt_start(model = model, data = data, y = y, 
                             pstart = pstart, offset = offset, 
-                            fixed = fixed, weights = weights)
+                            fixed = s$fixed, weights = weights)
     }
 
     args <- list()
@@ -766,7 +810,7 @@ mlt <- function(model, data, weights = NULL, offset = NULL, fixed = NULL,
     args$offset <- offset
     args$theta <- theta
     args$subset <- NULL ### only available in update()
-    args$scale <- scale
+    args$scaleparm <- scaleparm
     args$optim <- optim
     args$fixed <- fixed
     ret <- do.call(".mlt_fit", args)
@@ -790,7 +834,7 @@ update.mlt_fit <- function(object, weights = stats::weights(object),
         strt <- strt[!names(strt) %in% names(fixed)]
         return(mlt(object$model, data = object$data, weights = weights,
             offset = offset, theta = strt, fixed = fixed,
-            scale = object$scale, optim = object$optim))
+            scaleparm = object$scaleparm, optim = object$optim))
     }
 
     stopifnot(length(weights) == NROW(object$data))
@@ -810,7 +854,7 @@ update.mlt_fit <- function(object, weights = stats::weights(object),
     args$subset <- subset
     args$offset <- offset
     args$theta <- theta
-    args$scale <- object$scale
+    args$scaleparm <- object$scaleparm
     args$optim <- object$optim
     ret <- do.call(".mlt_fit", args)
     ret$call <- match.call()
@@ -832,7 +876,7 @@ fmlt <- function(object, frailty = c("Gamma", "InvGauss", "PositiveStable"),
                weights = weights(object),
                offset = object$offset, 
                theta = coef(object), fixed = object$fixed, 
-               scale = object$scale, optim = object$optim)
+               scaleparm = object$scaleparm, optim = object$optim)
     class(ret) <- c("fmlt", class(ret))
     ret
 }
@@ -856,7 +900,7 @@ cmlt <- function(object, interval = fr$support, ...) {
     ret <- mlt(model = model, data = object$data, weights = weights(object),
                offset = object$offset, 
                theta = coef(object), fixed = object$fixed, 
-               scale = object$scale, optim = object$optim)
+               scaleparm = object$scaleparm, optim = object$optim)
     class(ret) <- c("fmlt", class(ret))
     ret
 }
